TensorStorage

Struct TensorStorage 

pub struct TensorStorage<T, A: TensorAllocator> { /* private fields */ }
Definition of the buffer for a tensor.

Implementations

Source

impl<T, A: TensorAllocator> TensorStorage<T, A>

pub fn as_ptr(&self) -> *const T

Returns the pointer to the tensor memory.

pub fn as_mut_ptr(&mut self) -> *mut T

Returns the pointer to the tensor memory.

pub fn as_slice(&self) -> &[T]

Returns the data pointer as a slice.

pub fn as_mut_slice(&mut self) -> &mut [T]

Returns the data pointer as a mutable slice.

pub fn len(&self) -> usize

Returns the number of bytes contained in this TensorStorage.

pub fn is_empty(&self) -> bool

Returns true if the TensorStorage has a length of 0.

pub fn layout(&self) -> Layout

Returns the layout of the tensor buffer.

pub fn alloc(&self) -> &A

Returns the allocator of the tensor buffer.

pub fn from_vec(value: Vec<T>, alloc: A) -> Self

Creates a new tensor buffer from a vector.

pub unsafe fn from_raw_parts(data: *const T, len: usize, alloc: A) -> Self

Creates a new tensor buffer from a raw pointer.

Safety

The pointer must be non-null and the length must be valid.

pub fn into_vec(self) -> Vec<T>

Converts the TensorStorage into a Vec<T>.

Returns Err(self) if the buffer does not have the same layout as the destination Vec.