TensorError

Enum TensorError 

pub enum TensorError {
    CastError,
    InvalidShape(usize),
    IndexOutOfBounds(usize),
    StorageError(TensorAllocatorError),
    DimensionMismatch(String),
    UnsupportedOperation(String),
}
An error type for tensor operations.

Variants



CastError

Error when the cast operation fails.



InvalidShape(usize)

The number of elements in the data does not match the shape of the tensor.



IndexOutOfBounds(usize)

Index out of bounds.



StorageError(TensorAllocatorError)

Error with the tensor storage.



DimensionMismatch(String)

Dimension mismatch for operations requiring compatible shapes.



UnsupportedOperation(String)

Unsupported operation for the given data type or tensor configuration.